<?php
/**
 * TPC! Memory Usage Classes
 * 
 * @package TPC_Memory_Usage
 * @subpackage Core
 */

if (!defined('ABSPATH'))
	die('-1');

class tpcmem {
	/**
	 * Instance of TPC! Memory Usage logger object
	 * 
	 * @var tpcmem_log
	 */
	protected static $_instance = null;
	
	/**
	 * Memory usage variable
	 * 
	 * @var array
	 */
	protected $_usageData = array();
	
	/**
	 * Last recorded memory usage
	 * 
	 * @var int
	 */
	protected $_lastRecorded = null;
	
	/**
	 * Peak memory usage for current script
	 * 
	 * @var array
	 */
	protected $_peakMemoryUsage = null;
	
	/**
	 * Constructor
	 * 
	 * Instantiate using {@link getInstance()}; tpcmem_log is a
	 * singleton object.
	 * 
	 * @return void
	 */
	protected function __construct() {
	}
	
	/**
	 * Enforce singleton; disallow cloning
	 *
	 * @return void
	 */
	private function __clone() {
	}
	
	/**
	 * Singleton instance
	 *
	 * @return tpcmem_log
	 */
	public function getInstance() {
		if (null === self::$_instance)
			self::$_instance = new self();
		
		return self::$_instance;
	}
	
	public function record(Array $data) {
		if (!function_exists('memory_get_usage'))
			return false;
		
		if (!isset($data['hook']))
			$data['hook'] = 'unknown';
		
		if (!isset($data['action']))
			$data['action'] = 'Unknown';
		
		if (!isset($data['time']))
			$data['time'] = time();
		
		if (!isset($data['usage']))
			$data['usage'] = $this->current();
		
		// Log memory usage data
		$this->_usageData[] = $data;
		
		if (!is_array($this->_peakMemoryUsage) || $this->_peakMemoryUsage['usage'] < $data['usage'])
			$this->_peakMemoryUsage = $data;

		return $data;
	}
	
	/**
	 * Retrieve the number of times memory usage has been recorded.
	 *
	 * @return int|false Number of times memory usage has been recorded, or false on failure.
	 */
	public function count() {
		if (!is_array($this->_usageData))
			return false;
		
		$count = count($this->_usageData);
		
		return $count;
	}
	
	/**
	 * Returns the amount of memory allocated to PHP.
	 *
	 * @param bool Optional. Defaults to false. Set this to TRUE to get the real size of memory allocated from the system. If not set or FALSE only the memory used by emalloc() is reported.
	 * @return int The amount of memory, in bytes, that's been allocated to this PHP script.
	 */
	public function current($realUsage = false) {
		if (!function_exists('memory_get_usage'))
			return false;
		
		$usage = memory_get_usage($realUsage);
		
		return $usage;
	}
	
	/**
	 * Returns the peak of memory allocated by PHP.
	 * 
	 * @param bool Optional. Defaults to false. Set this to TRUE to get the real size of memory allocated from the system. If not set or FALSE only the memory used by emalloc() is reported.
	 * @return int The peak of memory, in bytes, that's been allocated to this PHP script.
	 */
	public function peak($realUsage = false) {
		if (!function_exists('memory_get_peak_usage'))
			return false;
		
		$usage = memory_get_peak_usage($realUsage);
		
		return $usage;
	}
	
	/**
	 * Retrieve the peak memory usage for the current script.
	 * 
	 * @return array|null
	 */
	public function scriptPeak() {
		return $this->_peakMemoryUsage;
	}
	
	/**
	 * Get the global memory limit (lowest of WP_MEMORY_LIMIT and php.ini 'memory_limit' directive).
	 * 
	 * @param string|bool $suffix No suffix by default. Adds a suffix to the returned string.
	 * @return int|string The global memory limit.
	 */
	public function getLimit($suffix = false) {
		$phpLimit = (int) str_replace('M', '', $this->phpLimit());
		$wpLimit = (int) str_replace('M', '', WP_MEMORY_LIMIT);
		
		$limit = ($phpLimit < $wpLimit) ? $phpLimit : $wpLimit;
		
		if ($suffix)
			return $limit . $suffix;
		
		return $limit;
	}
	
	/**
	 * Retrieves PHP memory limit from 'memory_limit' php.ini directive.
	 * 
	 * @return string The PHP memory limit
	 */
	public function phpLimit() {
		return @ini_get('memory_limit');
	}
}